import { Link } from "react-router-dom";
import { Message, User, Channel } from "../types";

import "../styles/MessageComponent.css";

export default function MessageComponent({ message, user, channel, deleteMessage }: {
        message: Message;
        user: User | undefined;
        channel: Channel | undefined;
        deleteMessage: ((messageId: number) => void) | undefined;
    }) {

    function getMessageArray(message: Message) {
        const array = []
        var string = ''
        message.content.split(" ").forEach((word) => {
            if (word.startsWith("@")) {
                const mention = message.mentions.find((mention) => `@${mention.username}` === word);
                if (mention) {
                    array.push(string)
                    string = ''
                    array.push(word)
                } else {
                    string += word + " "
                }
            } else if (word.startsWith("https://") || word.startsWith("http://")) {
                array.push(string)
                string = ''
                array.push(word)
            } else if (word.startsWith(":") && word.endsWith(":") && word.length > 2) {
                array.push(string)
                string = ''
                array.push(word)
            } else {
                string += word + " "
            }
        });
        array.push(string)
        return array.filter((word) => word !== '');
    }
    
    return (
        <div key={message.id} className="message">
            <div className="message-content">
                <img src={`/api/users/${message.username}/pfp`} alt="" className="message-user-pfp" />
                <div className="message-content-right">
                    <span><Link to={`/u/${message.username}`}>{message.username}</Link> {new Date(message.date * 1000).toLocaleString()}</span>
                    <div className="message-text">
                        {getMessageArray(message).map((word, index) => {
                            if (word.startsWith("@")) {
                                return <span key={index} ><Link to={`/u/${word.substring(1)}`}>{word}</Link> </span>
                            } else if (word.startsWith("https://") || word.startsWith("http://")) {
                                return <span key={index} ><Link to={word}>{word}</Link> </span>
                            } else if (word.startsWith(":") && word.endsWith(":") && word.length > 2) {
                                const emojiName = word.substring(1, word.length - 1);
                                return <span key={index} ><img src={`/api/emojis/${emojiName}`} alt={emojiName} className="message-emoji" /> </span>
                            }
                            return <span key={index}>{word} </span>;
                        })}
                    </div>
                </div>
            </div>
            <div>
                {message.content.toLocaleLowerCase().includes("gros cochon") && (
                    <img src="/pig.png" alt="Gros cochon" className="pig" />
                )}
            </div>
            <div>
                {message.content.toLocaleLowerCase().includes("le poisson steve") && (
                    <img src="/le_poisson_steve.png" alt="Le poisson steve" className="fish" />
                )}
            </div>
            {channel && deleteMessage ? (
                user?.id === message.user_id || user?.id === channel?.owner_id || user?.admin === 1) && (
                    <button onClick={() => {deleteMessage(message.id)}}>Delete</button>
                ) : (
                    <p>In <Link to={`/c/${message.channel_name}`}>{message.channel_name}</Link></p>
                )
            }
        </div>
    );
}